
#include "System.h"
#include "UDPPackets.h"
#include "UDPProtocol.h"
#include "UDPPackets.h"
#include "NetworkImplementation.h"
#include "pff.h"
#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000UL
#include <util/delay.h>

#define USE_SD_CARD

TChar DataBuffer[768];
TChar OwnMAC[6] = {0X00,0X26,0X2D,0X2D,0X26,0X00};
TChar OwnIP[4] = {169,254,24,37};
TChar GatewayMAC[6] = {0X00,0X26,0X2D,0XAC,0X47,0X96};
TChar GatewayIP[4] = {169,254,24,78};
#define CommPort 3416

struct TPacketInfo PacketInfo;
struct TPacketReader PacketReader;
struct TPacketWriter PacketWriter;

TFloat MyDivTime = 0.0f;
TInteger MyTime = 0;

/* File system object */
FATFS Fs;
/* Directory object */
DIR Dir;
/* File information */
FILINFO Fno;
/* Return value. Put this here to avoid bugs of avr-gcc */
WORD rb;

#define ActionCount 15

#define ActionStatus_None 0
#define ActionStatus_StartedFilling 1
#define ActionStatus_StartedEmptying 2

TInteger Actions[ActionCount];
TInteger CurrentAction;
TInteger CurrentActionStatus;

TBoolean ActuationOpenValve;
TBoolean ActuationOpenWater;

void MakePacketToGateway(void)
{
	CopyMemoryBuffer(PacketInfo.DestinationIP,GatewayIP,4);
	CopyMemoryBuffer(PacketInfo.SourceIP,OwnIP,4);
	PacketInfo.SourcePort = CommPort;
	PacketInfo.DestinationPort = CommPort;
}

void MakeUDPResponse(void)
{
	uint16_t i;
	for (i = 0;i < 4;i += 1)
	{
		PacketInfo.DestinationIP[i] = PacketInfo.SourceIP[i];
		PacketInfo.SourceIP[i] = OwnIP[i];
	}
	i = PacketInfo.DestinationPort;
	PacketInfo.DestinationPort = PacketInfo.SourcePort;
	PacketInfo.SourcePort = i;
}

void SendNoError(void)
{
	MakePacketToGateway();
	InitPacketWriter(&PacketInfo,&PacketWriter);
	WritePacket_StatusInformation(&PacketWriter,0,"\0",0);
	SendPacket(PacketWriter.Size);
}

void InitFile(void)
{
	while (1)
	{
		int res;
		unsigned char fn[20] = "udare.txt";
		if ((res = pf_open(fn)) == FR_OK)
		{
			SendNoError();
			break;
		}
		else
		{
			MakePacketToGateway();
			InitPacketWriter(&PacketInfo,&PacketWriter);
			WritePacket_StatusInformation(&PacketWriter,0,"Unable to open file.\0",21);
			SendPacket(PacketWriter.Size);
		}
		
		_delay_ms(100);
	}
}

void InitStorage(void)
{
#ifdef USE_SD_CARD

	while (pf_mount(&Fs) != FR_OK)
	{
		MakePacketToGateway();
		InitPacketWriter(&PacketInfo,&PacketWriter);
		WritePacket_StatusInformation(&PacketWriter,0,"Unable to initialize FAT.\0",26);
		SendPacket(PacketWriter.Size);

		_delay_ms(1000);
	}
	
	SendNoError();
	
	InitFile();

#endif
}

void ReadActionPlan(void)
{
#ifdef USE_SD_CARD

	TInteger i;
	PInteger Loc = ((PInteger)(GetDataLocation()));
	DWORD cnt,res;

	do
	{
		res = pf_read(GetDataLocation(),(ActionCount + 2) * sizeof(TInteger),&cnt);
	}
	while (res != FR_OK);

	CurrentAction = *Loc;
	Loc += 1;
	
	CurrentActionStatus = *Loc;
	Loc += 1;
	
	for (i = 0;i < ActionCount;i += 1)
	{
		Actions[i] = *Loc;
		Loc += 1;
	}
	
	InitFile();

/*	TInteger i,res;
	WORD cnt;
	
	while (1)
	{
		pf_lseek(0);

		res = pf_read(&CurrentAction,sizeof(TInteger),&cnt);
		if (res != FR_OK)
		{
			goto SendErrRead;
		}
		if (cnt != sizeof(TInteger))
		{
			goto ReInitData;
		}

		res = pf_read(&CurrentActionStatus,sizeof(TInteger),&cnt);
		if (res != FR_OK)
		{
			goto SendErrRead;
		}
		if (cnt != sizeof(TInteger))
		{
			goto ReInitData;
		}

		res = pf_read(&Actions[0],ActionCount * sizeof(TInteger),&cnt);
		if (res != FR_OK)
		{
			goto SendErrRead;
		}
		if (cnt != (ActionCount * sizeof(TInteger)))
		{
			goto ReInitData;
		}
		
		break;
		
		ReInitData:;
		CurrentAction = 0;
		CurrentActionStatus = 0;
		for (i = 0;i < ActionCount;i += 1)
		{
			Actions[i] = 0;
		}
		break;

		SendErrRead:;
		MakePacketToGateway();
		InitPacketWriter(&PacketInfo,&PacketWriter);
		WritePacket_StatusInformation(&PacketWriter,0,"Unable to read file.\0",21);
		SendPacket(PacketWriter.Size);
		_delay_ms(1000);
	}
	SendNoError();*/
#endif
}

void WriteActionPlan(void)
{
#ifdef USE_SD_CARD

	TInteger i;
	PInteger Loc = ((PInteger)(GetDataLocation()));
	DWORD cnt,res;

	*Loc = CurrentAction;
	Loc += 1;
	
	*Loc = CurrentActionStatus;
	Loc += 1;
	
	for (i = 0;i < ActionCount;i += 1)
	{
		*Loc = Actions[i];
		Loc += 1;
	}

	do
	{
		res = pf_write(GetDataLocation(),(ActionCount + 2) * sizeof(TInteger),&cnt);
	}
	while (res != FR_OK);

	_delay_ms(100);

	InitFile();
/*
	TInteger res;
	WORD cnt;
	
	while (1)
	{
		pf_lseek(0);

		res = pf_write(&CurrentAction,sizeof(TInteger),&cnt);
		if ((res != FR_OK) || (cnt != (sizeof(TInteger))))
		{
			goto SendErr;
		}

		res = pf_write(&CurrentActionStatus,sizeof(TInteger),&cnt);
		if ((res != FR_OK) || (cnt != (sizeof(TInteger))))
		{
			goto SendErr;
		}

		res = pf_write(&Actions[0],ActionCount * sizeof(TInteger),&cnt);
		if ((res != FR_OK) || (cnt != (ActionCount * sizeof(TInteger))))
		{
			goto SendErr;
		}

		break;

		SendErr:;
		MakePacketToGateway();
		InitPacketWriter(&PacketInfo,&PacketWriter);
		WritePacket_StatusInformation(&PacketWriter,0,"Unable to write file.\0",22);
		SendPacket(PacketWriter.Size);
		_delay_ms(1000);
	}
	SendNoError();
*/
#endif
}

void SendActionPlan(TInteger8 Marker)
{
	InitPacketWriter(&PacketInfo,&PacketWriter);
	WritePacket_GetIrrigationPlan(&PacketWriter,Marker,ActionCount,Actions);
	SendPacket(PacketWriter.Size);
}

void ClearActions(void)
{
	TInteger i;
	CurrentAction = 0;
	CurrentActionStatus = 0;
	for (i = 0;i < ActionCount;i += 1)
	{
		Actions[i] = 0;
	}
}

TInteger RearangeActionPlan(void)
{
	TInteger i,pos;
	pos = 0;
	for (i = 0;i < ActionCount;i += 1)
	{
		if (Actions[i] != 0)
		{
			Actions[pos] = Actions[i];
			pos += 1;
		}
	}
	for (i = pos;i < ActionCount;i += 1)
	{
		Actions[i] = 0;
	}
	return pos;
}

void AddActionToPlan(TInteger Act)
{
	TInteger pos = RearangeActionPlan();
	TInteger i,j;
	for (i = 0;i < pos;i += 1)
	{
		if (Act < Actions[i])
		{
			if (pos == ActionCount)
			{
				pos -= 1;
			}
			for (j = pos;j > i;j -= 1)
			{
				Actions[j] = Actions[j - 1];
			}
			Actions[i] = Act;
			return;
		}
		else
		{
			if (Act == Actions[i])
			{
				return;
			}
		}
	}
	if (pos < ActionCount)
	{
		Actions[pos] = Act;
	}
}

void RemoveActionFromPlan(TInteger Act)
{
	TInteger pos = RearangeActionPlan();
	TInteger i;
	for (i = 0;i < pos;i += 1)
	{
		if (Actions[i] == Act)
		{
			Actions[i] = 0;
			break;
		}
	}
	RearangeActionPlan();
}

void InitActuation(void)
{
	ActuationOpenValve = False;
	ActuationOpenWater = False;

	DDRC |= ((1 << PC6) | (1 << PC7));
	PORTC &= (~((1 << PC6) | (1 << PC7)));
	
	DDRC |= ((1 << PC0) | (1 << PC1) | (1 << PC2));
	PORTC |= ((1 << PC0) | (1 << PC1) | (1 << PC2));
}

void OpenValve(void)
{
	ActuationOpenValve = True;
	PORTC |= (1 << PC7);
}

void CloseValve(void)
{
	ActuationOpenValve = False;
	PORTC &= (~(1 << PC7));
}

void OpenWater(void)
{
	ActuationOpenWater = True;
	PORTC |= (1 << PC6);
}

void CloseWater(void)
{
	ActuationOpenWater = False;
	PORTC &= (~(1 << PC6));
}

ISR(TIMER1_COMPA_vect)
{
	MyDivTime += 0.03125f;
	while (MyDivTime > 0)
	{
		MyDivTime -= 1.0f;
		MyTime += 1;
	}
}

void InitRunStartLED(void)
{
	DDRD |= (1 << PD7);
	PORTD |= (1 << PD7);
}

void InitTimer(void)
{
	// modul CTC - capitolul 16 din datasheet, 
	// sectiunea Register description- TCCR1B
	TCCR1B |= (1 << WGM12);

	// toggle pe OC1B la fiecare compare match
	// (de fiecare data cand contorul = OCR1A
	TCCR1A |= (1 << COM1B0);

	// prescaler 256 - capitolul 16
	// sectiunea Register Description - TCCR1B
	TCCR1B |= (1 << CS12);

	// 16000000 / 2 / 256 = 31250
	OCR1A = 1952;

	//Activeaza intreruperea 1 pentru timer
	TIMSK1 |= (1 << OCIE1A);
  
	sei();
}

void InitSensors(void)
{
	DDRA = 0;
}

uint8_t ReadSensors(void)
{
	return (~PINA);
}

void HandlePacket(void)
{
//Pachet cu protocolul meu
	if (PacketInfo.DestinationPort == CommPort)
	{
//Pregatesc raspunsul
		MakeUDPResponse();

//Verfic pachetul sa fie corect
		PacketReader.Data = GetDataLocation();
		if (PacketReadStart(&PacketReader) == False)
		{
			return;
		}
		if (VerifyPacketDevice(&PacketReader) == False)
		{
			return;
		}
		PacketReadRestart(&PacketReader);

		int8_t OpCode;
		PacketReadInteger8(&PacketReader,&OpCode);
		switch (OpCode)
		{
			case PacketOpcode_RequestStatus :
			{
				ReadPacket_RequestStatus(&PacketReader);

				InitPacketWriter(&PacketInfo,&PacketWriter);
				WritePacket_StatusResponse(&PacketWriter,PacketReader.Marker,ActuationOpenWater,ActuationOpenValve,ReadSensors(),MyTime,CurrentAction);
				SendPacket(PacketWriter.Size);
			}
			break;
			case PacketOpcode_SetTime :
			{
				ReadPacket_SetTime(&PacketReader,&MyTime);
			}
			break;
			case PacketOpcode_SetIrrigationTime :
			{
				TInteger NewTime;
				ReadPacket_SetIrrigationTime(&PacketReader,&NewTime);
				AddActionToPlan(NewTime);
				SendActionPlan(PacketReader.Marker);
				WriteActionPlan();
			}
			break;
			case PacketOpcode_GetIrrigationPlan :
			{
				SendActionPlan(PacketReader.Marker);
			}
			break;
			case PacketOpcode_DeleteIrrigationPlan :
			{
				TInteger OldTime;
				ReadPacket_DeleteIrrigationPlan(&PacketReader,&OldTime);
				RemoveActionFromPlan(OldTime);
				SendActionPlan(PacketReader.Marker);
				WriteActionPlan();
			}
			break;
		};
	}
}

void HandleActionsStart(void)
{
	if (CurrentAction > 0)
	{
		switch (CurrentActionStatus)
		{
			case ActionStatus_StartedFilling :
			{
				CloseValve();
				OpenWater();
			}
			break;
			case ActionStatus_StartedEmptying :
			{
				CloseWater();
				OpenValve();
			}
			break;
		}
	}
	else
	{
		CloseWater();
		CloseValve();
	}
}

void HandleActions(void)
{
	if (CurrentAction > 0)
	{
		TInteger8 Sensor = ReadSensors();
		switch (CurrentActionStatus)
		{
			case ActionStatus_StartedFilling :
			{
				if ((Sensor & 0XC0) != 0) // 1100 0000
				{
					CurrentActionStatus = ActionStatus_StartedEmptying;
					CloseWater();
					OpenValve();
					WriteActionPlan();
				}
			}
			break;
			case ActionStatus_StartedEmptying :
			{
				if (Sensor == 0)
				{
					CloseValve();
					CloseWater();
					CurrentAction = 0;
					CurrentActionStatus = ActionStatus_None;
					WriteActionPlan();
				}
			}
			break;
		};
	}
	if (CurrentAction == 0)
	{
		if ((Actions[0] > 0) && (MyTime >= Actions[0]))
		{
			CurrentAction = Actions[0];
			RemoveActionFromPlan(CurrentAction);
			CurrentActionStatus = ActionStatus_StartedFilling;
			CloseValve();
			OpenWater();

			WriteActionPlan();
			SendActionPlan(PacketReader.Marker);
		}
	}
}

int main(void)
{
	InitRunStartLED();
	
	InitActuation();

	InitSensors();

	InitTimer();
	
	ClearActions();

	InitNetwork(DataBuffer,OwnMAC,OwnIP,GatewayMAC);

	InitStorage();
	
	ReadActionPlan();
	
	HandleActionsStart();

/* Testarte ethernet + card sd
	while (1)
	{
		WORD cnt1,cnt2,res1,res2,res3,res4,res5;
		cnt1 = cnt2 = res1 = res2 = res3 = res4 = res5 = 999;

		unsigned char fn[20] = "udare.txt";
		do
		{
			res5 = pf_open(fn);
		}
		while (res5 != FR_OK);

		sprintf(GetDataLocation() + 136,"%d %d %d %d %d %d %d",cnt1,cnt2,res5,res1,res2,res3,res4);

		SendPacket(180);
		
		_delay_ms(500);

		do
		{
			res1 = pf_lseek(0);
		}
		while (res1 != FR_OK);

		sprintf(GetDataLocation() + 136,"%d %d %d %d %d %d %d",cnt1,cnt2,res5,res1,res2,res3,res4);

		SendPacket(180);
		
		_delay_ms(500);

		do
		{
			res2 = pf_read(GetDataLocation(),136,&cnt1);
		}
		while (res2 != FR_OK);

		sprintf(GetDataLocation() + 136,"%d %d %d %d %d %d %d",cnt1,cnt2,res5,res1,res2,res3,res4);

		SendPacket(180);
		
		_delay_ms(500);

		do
		{
			res3 = pf_lseek(0);
		}
		while (res3 != FR_OK);

		sprintf(GetDataLocation() + 136,"%d %d %d %d %d %d %d",cnt1,cnt2,res5,res1,res2,res3,res4);

		SendPacket(180);
		
		_delay_ms(500);

		*(GetDataLocation()) = (~(*(GetDataLocation())));
		do
		{
			res4 = pf_write(GetDataLocation(),136,&cnt2);
		}
		while ((res4 != FR_OK) || (cnt2 != 136));

		_delay_ms(500);

		do
		{
			res5 = pf_open(fn);
		}
		while (res5 != FR_OK);

		sprintf(GetDataLocation() + 136,"%d %d %d %d %d %d %d",cnt1,cnt2,res5,res1,res2,res3,res4);

		SendPacket(180);
		
		_delay_ms(500);
	}*/
	
	while (1)
	{
		TBoolean Res = ReceivePacket(&PacketInfo,&PacketReader);
		if (Res != False)
		{
			HandlePacket();
		}

		_delay_ms(10);

		HandleActions();

		_delay_ms(10);
	}
	

    return 0;
}
