
#include "System.h"
#include "UDPPackets.h"
#include "NetworkImplementation.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>

#define F_CPU 16000000UL
#include <util/delay.h>

#include "enc28j60.h"

static PChar Buffer;

static TPointer DeviceMAC;
static TPointer DeviceIP;
static TPointer DeviceGatewayMAC;
static PPacketInfo DevicePacketInfo;

void ToBigEndian16(PChar Destination,TUnsignedInteger16 Value)
{
    Destination[0] = ((TChar)((Value >> 8) & 0XFF));
    Destination[1] = ((TChar)((Value >> 0) & 0XFF));
}

TUnsignedInteger16 FromBigEndian16(PChar Source)
{
    return (((TUnsignedInteger16)(Source[0])) << 8) | ((TUnsignedInteger16)(Source[1]));
}

void MakeEthernetHeader(TBoolean IPv4)
{
//Locul unde este pus headerul Ethernet
    PChar EthernetHeader = Buffer;

//Copiaza MAC-urile
    CopyMemoryBuffer(EthernetHeader,DeviceGatewayMAC,6);
    CopyMemoryBuffer(EthernetHeader + 6,DeviceMAC,6);

//Seteaza protocolul
    if (IPv4 == False)
    {
//ARP
        ToBigEndian16(EthernetHeader + 12,0X0806);
    }
    else
    {
//IP
        ToBigEndian16(EthernetHeader + 12,0X0800);
    }
}

void MakeIPHeader(TBoolean UDP)
{
//Locul unde este pus headerul IP
    PChar IPHeader = Buffer + 14;

//Versiunea IP
    *((PUnsignedInteger8)(IPHeader)) = 0X45;
//Detalii
    *((PUnsignedInteger8)(IPHeader + 1)) = 0X00;

//Identificare
    ToBigEndian16(IPHeader + 4,0X0000);
//Flag-urile, Fragmente
    ToBigEndian16(IPHeader + 6,0X4000);

//TTL
    *((PUnsignedInteger8)(IPHeader + 8)) = 64;
//Protocol
    if (UDP == False)
    {
//ICMP
        *((PUnsignedInteger8)(IPHeader + 9)) = 0X01;
    }
    else
    {
//UDP
        *((PUnsignedInteger8)(IPHeader + 9)) = 0X11;
    }

//Checksum vid
    ToBigEndian16(IPHeader + 10,0X0000);

//Adresele sursa si destinatie
    CopyMemoryBuffer(IPHeader + 12,DevicePacketInfo->SourceIP,4);
    CopyMemoryBuffer(IPHeader + 16,DevicePacketInfo->DestinationIP,4);
    
}

void MakeUDPHeader(void)
{
//Locul unde este pus headerul UDP
    PChar UDPHeader = Buffer + 34;

//Porturile sursa si destinatie
    ToBigEndian16(UDPHeader,DevicePacketInfo->SourcePort);
    ToBigEndian16(UDPHeader + 2,DevicePacketInfo->DestinationPort);
}

PChar GetDataLocation(void)
{
    return Buffer + 42;
}

void FinishUDPPacket(TUnsignedInteger16 Length)
{
//Locul unde este pus headerul UDP
    PChar UDPHeader = Buffer + 34;

//Lungimea
    ToBigEndian16(UDPHeader + 4,Length + 8);

//Checksum
    ToBigEndian16(UDPHeader + 6,0);
}

void FinishIPPacket(TUnsignedInteger16 Length)
{
//Locul unde este pus headerul IP
    PChar IPHeader = Buffer + 14;

//Lungimea
    ToBigEndian16(IPHeader + 2,Length + 20);

	unsigned short checksum;
//Calculeaza checksum
    {
		TUnsignedInteger32 sum = 0;
		TUnsignedInteger32 count = 20;
		PChar Addr = IPHeader;

		while (count > 1)
		{
			sum += FromBigEndian16(Addr);
			count -= 2;
			Addr += 2;
		}

		if (count > 0)
		{
			sum += ((*((PUnsignedInteger8)(Addr))) << 8);
		}

		while (sum >> 16)
		{
			sum = (sum & 0xffff) + (sum >> 16);
		}

		checksum = ~sum;
    }

//Checksum
   ToBigEndian16(IPHeader + 10,checksum);
}

void StartUDPPacket(void)
{
    //
}

void InitNetwork(TPointer DataBuffer,TPointer OwnMAC,TPointer OwnIP,TPointer GatewayMAC)
{
    Buffer = ((PChar)(DataBuffer));
    DeviceMAC = OwnMAC;
    DeviceIP = OwnIP;
    DeviceGatewayMAC = GatewayMAC;

	DDRC = 0X3;

    // Set output to gnd, reset the ethernet chip.
    PORTC &= ~(1<<PC0);
    _delay_ms(300);

    // Set output to Vcc, reset inactive.
    PORTC |= (1<<PC0);
    _delay_ms(300);

    //initializare ENC28J60
	enc28j60Init((uint8_t *)DeviceMAC);
    _delay_ms(300);

    enc28j60PhyWrite(PHLCON,0x3216);
    _delay_ms(300);
	
	_delay_ms(5000);

}

void InitPacketWriter(PPacketInfo PacketInformation,PPacketWriter Writer)
{
	DevicePacketInfo = PacketInformation;

    MakeEthernetHeader(True);
    MakeIPHeader(True);
    MakeUDPHeader();

	Writer->Data = GetDataLocation();
	PacketWriteStart(Writer);
}

void SendPacket(TUnsignedInteger16 Len)
{
	if (Len == 0)
	{
		return ;
	}
/*    PChar Loc;
	
	struct TPacketInfo INFO;
	DevicePacketInfo = &INFO;

    DevicePacketInfo->SourceIP[0] = 169;
    DevicePacketInfo->SourceIP[1] = 254;
    DevicePacketInfo->SourceIP[2] = 24;
    DevicePacketInfo->SourceIP[3] = 37;

    DevicePacketInfo->DestinationIP[0] = 169;
    DevicePacketInfo->DestinationIP[1] = 254;
    DevicePacketInfo->DestinationIP[2] = 24;
    DevicePacketInfo->DestinationIP[3] = 78;

    DevicePacketInfo->SourcePort = 4000;
    DevicePacketInfo->DestinationPort = 5000;

    MakeEthernetHeader(True);
    MakeIPHeader(True);
    MakeUDPHeader();

    Loc = GetDataLocation();*/
	
/*	if (Len == 0)
	{
		Len += 400;
		
		int i;
		for (i = 0;i < 400;i += 1)
		{
			Loc[i] = 'a';
		}
//		Loc[0] = 'N';
//		Loc[1] = 'o';
//		Loc[2] = ' ';
//		Loc[3] = 'd';
//		Loc[4] = 'a';
//		Loc[5] = 't';
//		Loc[6] = 'a';
//		Loc[7] = '.';
	}*/

/*    Loc[0] = 'H';
    Loc[1] = 'e';
    Loc[2] = 'l';
    Loc[3] = 'l';
    Loc[4] = 'o';
    Loc[5] = 0;
	int i;
	for (i = 'A';i <= 'Z';i += 1)
	{
	    Loc[6 + i - 'A'] = (rand() % 26) + 'A';
	}

    Len = 6 + 26;*/

/*    PChar Loc;
	
	struct TPacketInfo INFO;
	DevicePacketInfo = &INFO;

    DevicePacketInfo->SourceIP[0] = 169;
    DevicePacketInfo->SourceIP[1] = 254;
    DevicePacketInfo->SourceIP[2] = 24;
    DevicePacketInfo->SourceIP[3] = 37;

    DevicePacketInfo->DestinationIP[0] = 169;
    DevicePacketInfo->DestinationIP[1] = 254;
    DevicePacketInfo->DestinationIP[2] = 24;
    DevicePacketInfo->DestinationIP[3] = 78;

    DevicePacketInfo->SourcePort = 3416;
    DevicePacketInfo->DestinationPort = 3416;

    MakeEthernetHeader(True);
    MakeIPHeader(True);
    MakeUDPHeader();

    Loc = GetDataLocation();*/

    FinishUDPPacket(Len);
    FinishIPPacket(Len + 8);

    //trimitere efectiva
    enc28j60PacketSend(Len + 8 + 20 + 14,Buffer);
	
	_delay_ms(100);
}

TBoolean ReceivePacket(PPacketInfo PacketInformation,PPacketReader Reader)
{
    TUnsignedInteger16 Len = enc28j60PacketReceive(768,Buffer);

	if (Len > 0)
	{
		int i;

//Locul unde este pus headerul IP
		PChar IPHeader = Buffer + 14;
//Locul unde este pus headerul UDP
		PChar UDPHeader = Buffer + 34;
		
		for (i = 0;i < 4;i += 1)
		{
			if ((IPHeader + 16)[i] != ((PChar)(DeviceIP))[i])
			{
				return False;
			}
		}

//Adresele sursa si destinatie
		CopyMemoryBuffer(PacketInformation->SourceIP,IPHeader + 12,4);
		CopyMemoryBuffer(PacketInformation->DestinationIP,IPHeader + 16,4);

//Porturile sursa si destinatie
		PacketInformation->SourcePort = FromBigEndian16(UDPHeader);
		PacketInformation->DestinationPort = FromBigEndian16(UDPHeader + 2);

		return True;
	}
    return False;
}
