
#include "System.h"
#include "UDPPackets.h"

void PacketWriteStart(PPacketWriter Packet)
{
    Packet->Size = sizeof(TInteger16);
}

void PacketWriteFinish(PPacketWriter Packet,TInteger8 Marker)
{
    TInteger16 Size = ((TInteger16)(Packet->Size));
    TInteger16 Count = Size / (PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8));
    TInteger16 i;
    if ((Size % (PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8))) > 0)
    {
        Count += 1;
    }
    for (i = Count - 1;i >= 0;i -= 1)
    {
        CopyMemoryBuffer(((PChar)(Packet->Data)) + (i * PACKET_CRYPT_BUFFER_SIZE),
                         ((PChar)(Packet->Data)) + (i * (PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8))),
                         PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8));
        *((PInteger8)(((PChar)(Packet->Data)) + (((i + 1) * PACKET_CRYPT_BUFFER_SIZE) - 1))) = Marker;
    }
    *((PInteger16)(Packet->Data)) = Packet->Size;

    Packet->Size += Count;
    while ((Packet->Size % PACKET_CRYPT_BUFFER_SIZE) != 0)
    {
        Packet->Size += 1;
    }
}

TInteger16 PacketGetRemainingSize(PPacketWriter Packet)
{
    return PACKET_MAX_AVAILABLE_SIZE - Packet->Size;
}

TBoolean PacketWriteInteger8(PPacketWriter Packet,TInteger8 Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TInteger8));
}

TBoolean PacketWriteInteger16(PPacketWriter Packet,TInteger16 Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TInteger16));
}

TBoolean PacketWriteInteger32(PPacketWriter Packet,TInteger32 Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TInteger32));
}

TBoolean PacketWriteInteger64(PPacketWriter Packet,TInteger64 Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TInteger64));
}

TBoolean PacketWriteInteger(PPacketWriter Packet,TInteger Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TInteger));
}

TBoolean PacketWriteFloat32(PPacketWriter Packet,TFloat32 Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TFloat32));
}

TBoolean PacketWriteFloat64(PPacketWriter Packet,TFloat64 Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TFloat64));
}

TBoolean PacketWriteFloat(PPacketWriter Packet,TFloat Value)
{
    return PacketWriteRawBuffer(Packet,&Value,sizeof(TFloat));
}

TBoolean PacketWriteString(PPacketWriter Packet,PChar String)
{
    return PacketWriteStringLen(Packet,String,((TInteger16)(StringLength(String))));
}

TBoolean PacketWriteStringLen(PPacketWriter Packet,PChar String,TInteger16 Length)
{
    return PacketWriteBuffer(Packet,String,Length * sizeof(TChar));
}

TBoolean PacketWriteBuffer(PPacketWriter Packet,TPointer Buffer,TInteger16 Length)
{
    if (PacketWriteInteger16(Packet,((TInteger16)(Length))) == False)
    {
        return False;
    }
    return PacketWriteRawBuffer(Packet,Buffer,Length);
}

TBoolean PacketWriteRawBuffer(PPacketWriter Packet,TPointer Buffer,TInteger16 Length)
{
    if (PacketGetRemainingSize(Packet) < Length)
    {
        return False;
    }
    CopyMemoryBuffer(((PChar)(Packet->Data)) + Packet->Size,Buffer,Length);
    Packet->Size += ((TInteger16)(Length));
    return True;
}

TBoolean PacketReadStart(PPacketReader Packet)
{
    TInteger16 Size,Count,i;

    Packet->Pos = sizeof(TInteger16);
    Packet->Size = *((PInteger16)(Packet->Data));
    Packet->Marker = *(((PChar)(Packet->Data)) + (PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8)));

    Size = Packet->Size;

    Count = Size / (PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8));
    if ((Size % (PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8))) > 0)
    {
        Count += 1;
    }

    for (i = 0;i < Count;i += 1)
    {
        if (*(((PChar)(Packet->Data)) + ((PACKET_CRYPT_BUFFER_SIZE * (i + 1)) - 1)) != Packet->Marker)
        {
            return False;
        }
    }

    for (i = 0;i < Count;i += 1)
    {
        CopyMemoryBuffer(((PChar)(Packet->Data)) + (i * (PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8))),
                         ((PChar)(Packet->Data)) + (i * PACKET_CRYPT_BUFFER_SIZE),
                         PACKET_CRYPT_BUFFER_SIZE - sizeof(TInteger8));
    }

    return True;
}

void PacketReadRestart(PPacketReader Packet)
{
    Packet->Pos = sizeof(TInteger16);
}

TBoolean PacketReadInteger8(PPacketReader Packet,PInteger8 Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TInteger8));
}

TBoolean PacketReadInteger16(PPacketReader Packet,PInteger16 Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TInteger16));
}

TBoolean PacketReadInteger32(PPacketReader Packet,PInteger32 Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TInteger32));
}

TBoolean PacketReadInteger64(PPacketReader Packet,PInteger64 Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TInteger64));
}

TBoolean PacketReadInteger(PPacketReader Packet,PInteger Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TInteger));
}

TBoolean PacketReadFloat32(PPacketReader Packet,PFloat32 Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TFloat32));
}

TBoolean PacketReadFloat64(PPacketReader Packet,PFloat64 Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TFloat64));
}

TBoolean PacketReadFloat(PPacketReader Packet,PFloat Value)
{
    return PacketReadRawBuffer(Packet,Value,sizeof(TFloat));
}

TBoolean PacketReadString(PPacketReader Packet,PChar *String)
{
    TInteger16 Length;
    return PacketReadStringLen(Packet,String,&Length);
}

TBoolean PacketReadStringLen(PPacketReader Packet,PChar *String,PInteger16 Length)
{
    PPointer Buffer = ((PPointer)(String));
    return PacketReadBuffer(Packet,Buffer,Length);
}

TBoolean PacketReadBuffer(PPacketReader Packet,PPointer Buffer,PInteger16 Length)
{
    if (PacketReadInteger16(Packet,Length) == False)
    {
        return False;
    }
    if ((Packet->Size - Packet->Pos) < (*Length))
    {
        return False;
    }
    (*Buffer) = ((TPointer)(((PChar)(Packet->Data)) + Packet->Pos));
    Packet->Pos += (*Length);
    return True;
}

TBoolean PacketReadRawBuffer(PPacketReader Packet,TPointer Buffer,TInteger16 Length)
{
    if ((Packet->Size - Packet->Pos) < Length)
    {
        return False;
    }
    CopyMemoryBuffer(Buffer,((PChar)(Packet->Data)) + Packet->Pos,Length);
    Packet->Pos += Length;
    return True;
}
