
#include "System.h"
#include "UDPPackets.h"
#include "UDPProtocol.h"

void WritePacket_StatusInformation(PPacketWriter Packet,TInteger8 Marker,PChar Info,TInteger Length)
{
    PacketWriteInteger8(Packet,PacketOpcode_StatusInformation);
    PacketWriteStringLen(Packet,Info,((TInteger16)(Length)));
    PacketWriteFinish(Packet,Marker);
}

void WritePacket_SetTime(PPacketWriter Packet,TInteger8 Marker,TInteger Time)
{
    PacketWriteInteger8(Packet,PacketOpcode_SetTime);
    PacketWriteInteger(Packet,Time);
    PacketWriteFinish(Packet,Marker);
}

void WritePacket_RequestStatus(PPacketWriter Packet,TInteger8 Marker)
{
    PacketWriteInteger8(Packet,PacketOpcode_RequestStatus);
    PacketWriteFinish(Packet,Marker);
}

void WritePacket_StatusResponse(PPacketWriter Packet,TInteger8 Marker,TBoolean WaterSourceActive,TBoolean ValveOpen,
                                TInteger8 Sensors,TInteger MyTime,TInteger CurrentActionTime)
{
    PacketWriteInteger8(Packet,PacketOpcode_StatusResponse);

    PacketWriteInteger8(Packet,WaterSourceActive);
    PacketWriteInteger8(Packet,ValveOpen);

    PacketWriteInteger8(Packet,Sensors);

    PacketWriteInteger(Packet,MyTime);
    PacketWriteInteger(Packet,CurrentActionTime);

    PacketWriteFinish(Packet,Marker);
}

void WritePacket_SetIrrigationTime(PPacketWriter Packet,TInteger8 Marker,TInteger NewTime)
{
    PacketWriteInteger8(Packet,PacketOpcode_SetIrrigationTime);
    PacketWriteInteger(Packet,NewTime);
    PacketWriteFinish(Packet,Marker);
}

void WritePacket_GetIrrigationPlan(PPacketWriter Packet,TInteger8 Marker,TInteger Count,PInteger Plan)
{
    TInteger i;

    PacketWriteInteger8(Packet,PacketOpcode_GetIrrigationPlan);

    PacketWriteInteger(Packet,Count);
    for (i = 0;i < Count;i += 1)
    {
        PacketWriteInteger(Packet,Plan[i]);
    }

    PacketWriteFinish(Packet,Marker);
}

void WritePacket_DeleteIrrigationPlan(PPacketWriter Packet,TInteger8 Marker,TInteger Time)
{
    PacketWriteInteger8(Packet,PacketOpcode_DeleteIrrigationPlan);
    PacketWriteInteger(Packet,Time);
    PacketWriteFinish(Packet,Marker);
}

TBoolean VerifyPacketDevice(PPacketReader Reader)
{
    TInteger8 OpCode;

    TBoolean Res = PacketReadInteger8(Reader,&OpCode);

    if (Res == False)
    {
        return False;
    }

    switch (OpCode)
    {
        case PacketOpcode_RequestStatus :
        {
            return VerifyPacket_RequestStatus(Reader);
        }
        break;
        case PacketOpcode_SetTime :
        {
            return VerifyPacket_SetTime(Reader);
        }
        break;
        case PacketOpcode_SetIrrigationTime :
        {
            return VerifyPacket_SetIrrigationTime(Reader);
        }
        break;
        case PacketOpcode_GetIrrigationPlan :
        {
            return VerifyPacket_GetIrrigationPlan(Reader);
        }
        break;
        case PacketOpcode_DeleteIrrigationPlan :
        {
            return VerifyPacket_DeleteIrrigationPlan(Reader);
        }
        break;
    };

    return False;
}

TBoolean VerifyPacketProgram(PPacketReader Reader)
{
    TInteger8 OpCode;

    TBoolean Res = PacketReadInteger8(Reader,&OpCode);

    if (Res == False)
    {
        return False;
    }

    switch (OpCode)
    {
        case PacketOpcode_StatusInformation :
        {
            return VerifyPacket_StatusInformation(Reader);
        }
        break;
        case PacketOpcode_StatusResponse :
        {
            return VerifyPacket_StatusResponse(Reader);
        }
        break;
        case PacketOpcode_GetIrrigationPlan :
        {
            return VerifyPacket_GetIrrigationPlan(Reader);
        }
        break;
    };

    return False;
}

TBoolean VerifyPacket_StatusInformation(PPacketReader Reader)
{
    TBoolean Res;

    PChar V;
    TInteger16 Len;

    Res = PacketReadStringLen(Reader,&V,&Len);
    if (Res == False)
    {
        return False;
    }

    if (Reader->Pos != Reader->Size)
    {
        return False;
    }
    return True;
}

TBoolean VerifyPacket_SetTime(PPacketReader Reader)
{
    TBoolean Res;

    TInteger V;    

    Res = PacketReadInteger(Reader,&V);
    if (Res == False)
    {
        return False;
    }

    if (Reader->Pos != Reader->Size)
    {
        return False;
    }
    return True;
}

TBoolean VerifyPacket_RequestStatus(PPacketReader Reader)
{
    if (Reader->Pos != Reader->Size)
    {
        return False;
    }
    return True;
}

TBoolean VerifyPacket_StatusResponse(PPacketReader Reader)
{
    TBoolean Res;

    TInteger8 V8;
    TInteger V;    

    Res = PacketReadInteger8(Reader,&V8);
    if (Res == False)
    {
        return False;
    }
    Res = PacketReadInteger8(Reader,&V8);
    if (Res == False)
    {
        return False;
    }

    Res = PacketReadInteger8(Reader,&V8);
    if (Res == False)
    {
        return False;
    }

    Res = PacketReadInteger(Reader,&V);
    if (Res == False)
    {
        return False;
    }

    Res = PacketReadInteger(Reader,&V);
    if (Res == False)
    {
        return False;
    }

    if (Reader->Pos != Reader->Size)
    {
        return False;
    }
    return True;
}

TBoolean VerifyPacket_SetIrrigationTime(PPacketReader Reader)
{
    TBoolean Res;

    TInteger V;    

    Res = PacketReadInteger(Reader,&V);
    if (Res == False)
    {
        return False;
    }

    if (Reader->Pos != Reader->Size)
    {
        return False;
    }
    return True;
}

TBoolean VerifyPacket_GetIrrigationPlan(PPacketReader Reader)
{
    TBoolean Res;

    TInteger V,VD,i;

    Res = PacketReadInteger(Reader,&V);
    if (Res == False)
    {
        return False;
    }

    for (i = 0;i < V;i += 1)
    {
        Res = PacketReadInteger(Reader,&VD);
        if (Res == False)
        {
            return False;
        }
    }

    if (Reader->Pos != Reader->Size)
    {
        return False;
    }
    return True;
}

TBoolean VerifyPacket_DeleteIrrigationPlan(PPacketReader Reader)
{
    TBoolean Res;

    TInteger V;    

    Res = PacketReadInteger(Reader,&V);
    if (Res == False)
    {
        return False;
    }

    if (Reader->Pos != Reader->Size)
    {
        return False;
    }
    return True;
}

void ReadPacket_StatusInformation(PPacketReader Reader,PChar *Info,PInteger Length)
{
    TInteger16 Len;
    PacketReadStringLen(Reader,Info,&Len);
    (*Length) = Len;
}

void ReadPacket_SetTime(PPacketReader Reader,PInteger Time)
{
    PacketReadInteger(Reader,Time);
}

void ReadPacket_RequestStatus(PPacketReader Reader)
{
}

void ReadPacket_StatusResponse(PPacketReader Reader,PBoolean WaterSourceActive,PBoolean ValveOpen,PInteger8 Sensors,
                               PInteger MyTime,PInteger CurrentActionTime)
{
    PacketReadInteger8(Reader,WaterSourceActive);
    PacketReadInteger8(Reader,ValveOpen);
    PacketReadInteger8(Reader,Sensors);
    PacketReadInteger(Reader,MyTime);
    PacketReadInteger(Reader,CurrentActionTime);
}

void ReadPacket_SetIrrigationTime(PPacketReader Reader,PInteger NewTime)
{
    PacketReadInteger(Reader,NewTime);
}

void ReadPacket_GetIrrigationPlan(PPacketReader Reader,PInteger Count,PInteger Plan)
{
    TInteger i;

    PacketReadInteger(Reader,Count);
    for (i = 0;i < (*Count);i += 1)
    {
        PacketReadInteger(Reader,Plan + i);
    }
}

void ReadPacket_DeleteIrrigationPlan(PPacketReader Reader,PInteger Time)
{
    PacketReadInteger(Reader,Time);
}
