
#include "System.h"
#include "UDPPackets.h"
#include "UDPProtocol.h"

#define SensorCount 8
#define ActionCount 15

#define MyIP "169.254.24.78"
#define DeviceIP "169.254.24.37"
#define CommPort 3416

struct TDeviceStatus;
typedef struct TDeviceStatus *PDeviceStatus;
struct TDeviceStatus
{
    TInteger DeviceTime;
    TInteger LastResponseTime;
    TInteger CurrentTime;
    TInteger LastUploadTime;

    TBoolean WaterSourceActive;
    TBoolean ValveOpen;

    TInteger8 WaterSensors;

    TInteger CurrentActionTime;

    //Program
    struct TPacketWriter PW;
    struct TPacketReader PR;
    TPacketData PD[1518];

    TInteger FrameCount;

    TBoolean ToExit;
    TBoolean ToUpdateTime;
    TBoolean ToWaitPlan;
    TBoolean WaitingPlan;
    TBoolean SendingActionAdd;
    TBoolean SendingActionDel;
    TInteger SendingActionInfo;

    TBoolean MenuMain;
    TBoolean MenuAddPlan;
    TBoolean MenuViewPlan;

    TInteger MenuSetPlanStep;
    TInteger MenuSetPlanValueYear;
    TInteger MenuSetPlanValueMonth;
    TInteger MenuSetPlanValueDay;
    TInteger MenuSetPlanValueHour;
    TInteger MenuSetPlanValueMinute;
    TInteger MenuSetPlanValueSecond;

    PChar Error;

    TInteger MenuSelectPlan;
    TInteger CurrentPlans[ActionCount];
};

static struct TDeviceStatus DeviceInfo;

void InitDevice(PDeviceStatus Device)
{
    Device->DeviceTime = -1;
    Device->LastResponseTime = -1;
    Device->CurrentTime = -1;
    Device->LastUploadTime = -1;

    Device->WaterSourceActive = False;
    Device->ValveOpen = False;
    Device->WaterSensors = 0;

    Device->CurrentActionTime = 0;

    Device->FrameCount = 0;

    Device->ToExit = False;
    Device->ToUpdateTime = False;
    Device->ToWaitPlan = False;
    Device->WaitingPlan = False;
    Device->SendingActionAdd = False;
    Device->SendingActionDel = False;
    Device->SendingActionInfo = 0;

    Device->PW.Data = Device->PD;
    Device->PR.Data = Device->PD;

    Device->MenuMain = True;
    Device->MenuAddPlan = False;
    Device->MenuViewPlan = False;

    DeviceInfo.Error = 0;
}

void TrimiteCerereDeInformatie(void)
{
    PacketWriteStart(&DeviceInfo.PW);

    WritePacket_RequestStatus(&DeviceInfo.PW,0);

    SendPacket(DeviceIP,CommPort,DeviceInfo.PW.Data,DeviceInfo.PW.Size);
}

void TrimiteNoulTimp(void)
{
    PacketWriteStart(&DeviceInfo.PW);

    WritePacket_SetTime(&DeviceInfo.PW,0,GetTime());

    SendPacket(DeviceIP,CommPort,DeviceInfo.PW.Data,DeviceInfo.PW.Size);
}

void TrimiteCererePlan(void)
{
    PacketWriteStart(&DeviceInfo.PW);

    WritePacket_GetIrrigationPlan(&DeviceInfo.PW,0,0,0);

    SendPacket(DeviceIP,CommPort,DeviceInfo.PW.Data,DeviceInfo.PW.Size);
}

void TrimiteAdaugarePlan(TInteger Plan)
{
    PacketWriteStart(&DeviceInfo.PW);

    WritePacket_SetIrrigationTime(&DeviceInfo.PW,0,Plan);

    SendPacket(DeviceIP,CommPort,DeviceInfo.PW.Data,DeviceInfo.PW.Size);
}

void TrimiteStergerePlan(TInteger Plan)
{
    PacketWriteStart(&DeviceInfo.PW);

    WritePacket_DeleteIrrigationPlan(&DeviceInfo.PW,0,Plan);

    SendPacket(DeviceIP,CommPort,DeviceInfo.PW.Data,DeviceInfo.PW.Size);
}

void HandleUploads(void)
{
    if (DeviceInfo.CurrentTime != DeviceInfo.LastUploadTime)
    {
        TrimiteCerereDeInformatie();
        DeviceInfo.LastUploadTime = DeviceInfo.CurrentTime;

        if (DeviceInfo.ToUpdateTime != False)
        {
            TrimiteNoulTimp();
            DeviceInfo.ToUpdateTime = False;
        }

        if (DeviceInfo.ToWaitPlan != False)
        {
            TrimiteCererePlan();
            DeviceInfo.WaitingPlan = True;
        }

        if (DeviceInfo.SendingActionAdd != False)
        {
            TrimiteAdaugarePlan(DeviceInfo.SendingActionInfo);
        }
        if (DeviceInfo.SendingActionDel != False)
        {
            TrimiteStergerePlan(DeviceInfo.SendingActionInfo);
        }
    }
}

void HandlePacket(PPacketReader PR)
{
    TInteger8 OpCode;

    if (PacketReadStart(PR) == False)
    {
        return;
    }
    if (VerifyPacketProgram(PR) == False)
    {
        return;
    }
    
    PacketReadRestart(PR);
    PacketReadInteger8(PR,&OpCode);

    switch (OpCode)
    {
        case PacketOpcode_StatusInformation :
        {
            PChar Err;
            TInteger Len;
            ReadPacket_StatusInformation(PR,&Err,&Len);

            if (Len > 0)
            {
                DeviceInfo.Error = ((PChar)(AllocateMemory(Len)));
                StringCopyN(DeviceInfo.Error,Err,Len - 1);
            }
            else
            {
                if (DeviceInfo.Error != 0)
                {
                    FreeMemory(DeviceInfo.Error);
                    DeviceInfo.Error = 0;
                }
            }
        }
        break;
        case PacketOpcode_StatusResponse :
        {
            ReadPacket_StatusResponse(PR,&DeviceInfo.WaterSourceActive,&DeviceInfo.ValveOpen,&DeviceInfo.WaterSensors,
                                      &DeviceInfo.DeviceTime,&DeviceInfo.CurrentActionTime);
        }
        break;
        case PacketOpcode_GetIrrigationPlan :
        {
            TInteger C;
            ReadPacket_GetIrrigationPlan(PR,&C,DeviceInfo.CurrentPlans);

            DeviceInfo.ToWaitPlan = False;
            DeviceInfo.WaitingPlan = False;

            DeviceInfo.MenuMain = False;
            DeviceInfo.MenuAddPlan = False;
            DeviceInfo.MenuViewPlan = True;

            DeviceInfo.MenuSelectPlan = -1;

            DeviceInfo.SendingActionAdd = False;
            DeviceInfo.SendingActionDel = False;
        }
        break;
    };
    DeviceInfo.LastResponseTime = GetTime();
}

void GetPacket(void)
{
#define DataLen 3000
    struct TPacketReader PR;
    TPacketData PD[DataLen];
    TChar SourceIP[20];
    TInteger16 SourcePort;
    TInteger Len;

    PR.Data = PD;

    while (1)
    {
        Len = DataLen;
        RecvPacket(SourceIP,&SourcePort,PR.Data,&Len);
        if (Len < 0)
        {
            break;
        }

        if ((StringCmp(SourceIP,DeviceIP) == 0) && (SourcePort == CommPort))
        {
            HandlePacket(&PR);
        }
    }
#undef DataLen
}

void HandleInput(PDeviceStatus Device)
{
    TKeyboardKey KK;
    TChar KC;

    KK = GetKeyPressed();

    if ((Device->ToWaitPlan != False) || (Device->WaitingPlan != False))
    {
        return;
    }

    if (KK != kkNone)
    {
        KC = KeyToChar(KK);
        if (KC != 0)
        {
            if (Device->MenuMain != False)
            {
                if ((KC == 'i') || (KC == 'I'))
                {
                    Device->ToExit = True;
                }
                if ((KC == 't') || (KC == 'T'))
                {
                    Device->ToUpdateTime = True;
                }
                if ((KC == 'o') || (KC == 'O'))
                {
                    TInteger TT = Device->CurrentTime + 60;
                    struct TTimeMoment TM;
                    GetMomentFromTime(&TM,TT);

                    Device->MenuMain = False;
                    Device->MenuAddPlan = True;
                    Device->MenuSetPlanStep = 0;

                    Device->MenuSetPlanValueYear = TM.Year;
                    Device->MenuSetPlanValueMonth = TM.Month;
                    Device->MenuSetPlanValueDay = TM.Day;
                    Device->MenuSetPlanValueHour = TM.Hour;
                    Device->MenuSetPlanValueMinute = TM.Minute;
                    Device->MenuSetPlanValueSecond = TM.Second;
                }
                if ((KC == 'p') || (KC == 'P'))
                {
                    Device->ToWaitPlan = True;
                }
                return;
            }
            if (Device->MenuAddPlan != False)
            {
                if ((KC >= '0') && (KC <= '9'))
                {
                    switch (Device->MenuSetPlanStep)
                    {
                        case 1:
                        {
                            TInteger NewValue = (Device->MenuSetPlanValueYear * 10) + (KC - '0');
                            if (NewValue < 2100)
                            {
                                Device->MenuSetPlanValueYear = NewValue;
                            }
                        }
                        break;
                        case 2:
                        {
                            TInteger NewValue = (Device->MenuSetPlanValueMonth * 10) + (KC - '0');
                            if (NewValue < 13)
                            {
                                Device->MenuSetPlanValueMonth = NewValue;
                            }
                        }
                        break;
                        case 3:
                        {
                            TInteger NewValue = (Device->MenuSetPlanValueDay * 10) + (KC - '0');
                            if (NewValue <= GetDaysInMonth(Device->MenuSetPlanValueYear,Device->MenuSetPlanValueMonth))
                            {
                                Device->MenuSetPlanValueDay = NewValue;
                            }
                        }
                        break;
                        case 4:
                        {
                            TInteger NewValue = (Device->MenuSetPlanValueHour * 10) + (KC - '0');
                            if (NewValue < 24)
                            {
                                Device->MenuSetPlanValueHour = NewValue;
                            }
                        }
                        break;
                        case 5:
                        {
                            TInteger NewValue = (Device->MenuSetPlanValueMinute * 10) + (KC - '0');
                            if (NewValue < 60)
                            {
                                Device->MenuSetPlanValueMinute = NewValue;
                            }
                        }
                        break;
                        case 6:
                        {
                            TInteger NewValue = (Device->MenuSetPlanValueSecond * 10) + (KC - '0');
                            if (NewValue < 60)
                            {
                                Device->MenuSetPlanValueSecond = NewValue;
                            }
                        }
                        break;
                    };
                }
                if (Device->MenuSetPlanStep != 7)
                {
                    if ((KC == 'a') || (KC == 'A'))
                    {
                        Device->MenuSetPlanStep = 1;
                    }
                    if ((KC == 'l') || (KC == 'L'))
                    {
                        Device->MenuSetPlanStep = 2;
                    }
                    if ((KC == 'z') || (KC == 'Z'))
                    {
                        Device->MenuSetPlanStep = 3;
                    }
                    if ((KC == 'o') || (KC == 'O'))
                    {
                        Device->MenuSetPlanStep = 4;
                    }
                    if ((KC == 'm') || (KC == 'M'))
                    {
                        Device->MenuSetPlanStep = 5;
                    }
                    if ((KC == 's') || (KC == 'S'))
                    {
                        Device->MenuSetPlanStep = 6;
                    }
                    if ((KC == 't') || (KC == 'T'))
                    {
                        struct TTimeMoment TM;
                        TM.Year = ((TInteger16)(Device->MenuSetPlanValueYear));
                        TM.Month = ((TInteger8)(Device->MenuSetPlanValueMonth));
                        TM.Day = ((TInteger8)(Device->MenuSetPlanValueDay));
                        TM.Hour = ((TInteger8)(Device->MenuSetPlanValueHour));
                        TM.Minute = ((TInteger8)(Device->MenuSetPlanValueMinute));
                        TM.Second = ((TInteger8)(Device->MenuSetPlanValueSecond));
                        if (VerifyTimeMoment(&TM) != False)
                        {
                            Device->SendingActionInfo = GetTimeFromMoment(&TM);
                            Device->SendingActionAdd = True;
                            Device->WaitingPlan = True;
                            Device->MenuSetPlanStep = 7;
                        }
                    }
                }
                if ((KC == 'r') || (KC == 'R'))
                {
                    Device->MenuMain = True;
                    Device->MenuAddPlan = False;
                    Device->WaitingPlan = False;
                }
                return;
            }
            if (Device->MenuViewPlan != False)
            {
                if ((KC == 'i') || (KC == 'I'))
                {
                    Device->MenuViewPlan = False;
                    Device->MenuMain = True;
                }
                if ((KC == 's') || (KC == 'S'))
                {
                    if (Device->MenuSelectPlan >= 0)
                    {
                        Device->SendingActionInfo = Device->CurrentPlans[Device->MenuSelectPlan];
                        Device->SendingActionDel = True;
                        Device->WaitingPlan = True;
                        Device->MenuSetPlanStep = 7;
                    }
                }
                if ((KC == 'q') || (KC == 'Q'))
                {
                    if (Device->MenuSelectPlan < 0)
                    {
                        Device->MenuSelectPlan = ActionCount - 1;
                    }
                    else
                    {
                        Device->MenuSelectPlan -= 1;
                        if (Device->MenuSelectPlan < 0)
                        {
                            Device->MenuSelectPlan = ActionCount - 1;
                        }
                    }
                    while ((Device->MenuSelectPlan >= 0) && (Device->CurrentPlans[Device->MenuSelectPlan] == 0))
                    {
                        Device->MenuSelectPlan -= 1;
                    }
                }
                if ((KC == 'a') || (KC == 'A'))
                {
                    if (Device->MenuSelectPlan < 0)
                    {
                        Device->MenuSelectPlan = 0;
                    }
                    else
                    {
                        Device->MenuSelectPlan += 1;
                        if (Device->MenuSelectPlan == ActionCount)
                        {
                            Device->MenuSelectPlan = 0;
                        }
                    }
                    while ((Device->MenuSelectPlan < ActionCount) && (Device->CurrentPlans[Device->MenuSelectPlan] == 0))
                    {
                        Device->MenuSelectPlan += 1;
                    }
                    if (Device->MenuSelectPlan == ActionCount)
                    {
                        Device->MenuSelectPlan = 0;
                        while ((Device->MenuSelectPlan < ActionCount) && (Device->CurrentPlans[Device->MenuSelectPlan] == 0))
                        {
                            Device->MenuSelectPlan += 1;
                        }
                        if (Device->MenuSelectPlan == ActionCount)
                        {
                            Device->MenuSelectPlan = -1;
                        }
                    }
                }
                return;
            }
        }
        else
        {
            if (KK == kkBackSpace)
            {
                if (Device->MenuAddPlan != False)
                {
                    switch (Device->MenuSetPlanStep)
                    {
                        case 1:
                        {
                            Device->MenuSetPlanValueYear /= 10;
                        }
                        break;
                        case 2:
                        {
                            Device->MenuSetPlanValueMonth /= 10;
                        }
                        break;
                        case 3:
                        {
                            Device->MenuSetPlanValueDay /= 10;
                        }
                        break;
                        case 4:
                        {
                            Device->MenuSetPlanValueHour /= 10;
                        }
                        break;
                        case 5:
                        {
                            Device->MenuSetPlanValueMinute /= 10;
                        }
                        break;
                        case 6:
                        {
                            Device->MenuSetPlanValueSecond /= 10;
                        }
                        break;
                    };
                }
            }
        }
    }
}

void DrawLineChars(TInteger XPos,TInteger YPos,TInteger Width,PChar BorderChar,PChar FillChar)
{
    TInteger a;
    GoToPosition(XPos,YPos);
    WriteString(BorderChar);

    for (a = 0;a < Width;a += 1)
    {
        WriteString(FillChar);
    }

    WriteString(BorderChar);
}

void DrawWaterLevelStatusBorder(TInteger BasePos,TInteger Width,TInteger Height)
{
    TInteger a;

    struct TConsoleColor CC;
    CC.Foreground = ccWhite;
    CC.Background = ccBlack;
    SetColor(&CC);

    for (a = 0;a < Height;a += 1)
    {
        DrawLineChars(BasePos,(a << 1) + 2,Width,"|"," ");
        DrawLineChars(BasePos,(a << 1) + 3,Width,"|"," ");
    }
    DrawLineChars(BasePos,(a << 1) + 2,Width,"+","-");
}

void DrawWaterLevelSensors(TInteger BasePos,TInteger Width,TInteger8 Status,TBoolean UseEmergency)
{
    TInteger a;

    struct TConsoleColor CC;

    for (a = 0;a < SensorCount;a += 1)
    {
        if ((Status & (1 << a)) != 0)
        {
            if (UseEmergency == False)
            {
                CC.Background = CC.Foreground = ccBlue;
            }
            else
            {
                TColor Colors[SensorCount] = {ccYellow,ccGreen,ccRed,ccRed,ccRed,ccGreen,ccYellow,ccRed};
                if ((DeviceInfo.FrameCount & 128) == 0)
                {
                    Colors[SensorCount - 1] = ccWhite;
                }
                else
                {
                    Colors[SensorCount - 1] = ccRed;
                }
                CC.Background = CC.Foreground = Colors[a];
            }

            SetColor(&CC);
            DrawLineChars(BasePos + 1,((SensorCount - a) << 1),Width,""," ");
            DrawLineChars(BasePos + 1,((SensorCount - a) << 1) + 1,Width,""," ");
        }
    }
}

TInteger GetPercentFill(TInteger8 Status)
{
    TInteger a;
    TFloat Level = 0;

    for (a = 0;a < SensorCount;a += 1)
    {
        if ((Status & (1 << a)) != 0)
        {
            Level += 1.0f / SensorCount;
        }
    }

    return ((TInteger)(Level * 100));
}

void WriteFillData(TInteger BasePos,TInteger8 Status)
{
    struct TConsoleColor CC;
    CC.Foreground = ccWhite;
    CC.Background = ccBlack;
    SetColor(&CC);

    GoToPosition(BasePos,(SensorCount << 1) + 3);
    WriteString("Umplut ");
    WriteInteger(GetPercentFill(Status));
    WriteString("%.");
}

void DrawWaterLevelStatus(TInteger8 Status)
{
    const TInteger BaseXPos = 55;

    const TInteger BasinWidth = 13;
    const TInteger StatusWidth = 2;
    const TInteger SpaceWidth = 1;
    const TInteger BorderWidth = 2;

    DrawWaterLevelStatusBorder(BaseXPos,StatusWidth,SensorCount);
    DrawWaterLevelStatusBorder(BaseXPos + StatusWidth + BorderWidth + SpaceWidth,BasinWidth,SensorCount);

    DrawWaterLevelSensors(BaseXPos + StatusWidth + BorderWidth + SpaceWidth,BasinWidth,Status,False);
    DrawWaterLevelSensors(BaseXPos,StatusWidth,Status,True);

    WriteFillData(BaseXPos,Status);    
}

void WriteTime(PChar S,TInteger Time)
{
    struct TTimeMoment TM;

    if (Time <= 0)
    {
        StringCopy(S,"Nici un timp.");
        return;
    }

    GetMomentFromTime(&TM,Time);

    S[0] = (TM.Day / 10) + '0';
    S[1] = (TM.Day % 10) + '0';
    S[2] = '/';

    S[3] = (TM.Month / 10) + '0';
    S[4] = (TM.Month % 10) + '0';
    S[5] = '/';

    S[6] = ((TM.Year / 1000) % 10) + '0';
    S[7] = ((TM.Year / 100) % 10) + '0';
    S[8] = ((TM.Year / 10) % 10) + '0';
    S[9] = ((TM.Year / 1) % 10) + '0';
    S[10] = ' ';

    S[11] = (TM.Hour / 10) + '0';
    S[12] = (TM.Hour % 10) + '0';
    S[13] = ':';

    S[14] = (TM.Minute / 10) + '0';
    S[15] = (TM.Minute % 10) + '0';
    S[16] = ':';

    S[17] = (TM.Second / 10) + '0';
    S[18] = (TM.Second % 10) + '0';
    S[19] = 0;
}

void DrawDeviceStatus(PDeviceStatus Status)
{
    TChar S[20];

    struct TConsoleColor CC;
    CC.Foreground = ccWhite;
    CC.Background = ccBlack;
    SetColor(&CC);

    GoToPosition(0,22);
    WriteString("Timpul dispozitivului : ");

    GoToPosition(24,22);
    WriteTime(S,Status->DeviceTime);
    WriteString(S);

    GoToPosition(0,23);
    WriteString("Ultima actualizare : ");

    GoToPosition(24,23);
    WriteTime(S,Status->LastResponseTime);
    WriteString(S);

    GoToPosition(0,24);
    WriteString("Timpul curent : ");

    GoToPosition(24,24);
    WriteTime(S,Status->CurrentTime);
    WriteString(S);

    GoToPosition(50,21);
    WriteString("Udarea curenta : ");
    GoToPosition(50,22);
    WriteTime(S,Status->CurrentActionTime);
    WriteString(S);

    GoToPosition(50,23);
    if (Status->WaterSourceActive != False)
    {
        WriteString("Pompa de apa este pornita.");
    }
    else
    {
        WriteString("Pompa de apa este oprita.");
    }

    GoToPosition(50,24);
    if (Status->ValveOpen != False)
    {
        WriteString("Valva este deschisa.");
    }
    else
    {
        WriteString("Valva este inchisa.");
    }

    GoToPosition(0,21);
    if (Status->Error != 0)
    {
        WriteString(Status->Error);
    }
    else
    {
        WriteString("Nici o eroare.");
    }

}

void SetMarkedColor(PConsoleColor CC,TInteger CurrentValue)
{
    if (DeviceInfo.MenuSetPlanStep == CurrentValue)
    {
        CC->Foreground = ccYellow;
        CC->Background = ccBlack;
        SetColor(CC);
    }
    else
    {
        CC->Foreground = ccWhite;
        CC->Background = ccBlack;
        SetColor(CC);
    }
}

void SetPlanSelectedColor(PConsoleColor CC,TInteger CurrentValue)
{
    if (DeviceInfo.MenuSelectPlan == CurrentValue)
    {
        CC->Foreground = ccYellow;
        CC->Background = ccBlack;
        SetColor(CC);
    }
    else
    {
        CC->Foreground = ccWhite;
        CC->Background = ccBlack;
        SetColor(CC);
    }
}

void DrawMenu(void)
{
    struct TConsoleColor CC;
    CC.Foreground = ccWhite;
    CC.Background = ccBlack;
    SetColor(&CC);

    if (DeviceInfo.MenuMain != False)
    {
        GoToPosition(5,5);
        WriteString("Actualizeaza timpul dispozitivului (T)");

        GoToPosition(5,6);
        WriteString("Stabilire ora de udare (O)");

        GoToPosition(5,7);
        WriteString("Vizualizare plan curent (P)");

        GoToPosition(5,8);
        WriteString("Iesire (I)");
    }

    if (DeviceInfo.MenuAddPlan != False)
    {
        GoToPosition(5,5);
        WriteString("Anul (A) : ");

        GoToPosition(5,6);
        WriteString("Luna (L) : ");

        GoToPosition(5,7);
        WriteString("Ziua (Z) : ");

        GoToPosition(5,8);
        WriteString("Ora (O) : ");

        GoToPosition(5,9);
        WriteString("Minutul (M) : ");

        GoToPosition(5,10);
        WriteString("Secunda (S) : ");

        GoToPosition(5,12);
        WriteString("Trimitere (T)");

        GoToPosition(5,13);
        WriteString("Renuntare (R)");

        SetMarkedColor(&CC,1);
        GoToPosition(19,5);
        WriteInteger(DeviceInfo.MenuSetPlanValueYear);

        SetMarkedColor(&CC,2);
        GoToPosition(19,6);
        WriteInteger(DeviceInfo.MenuSetPlanValueMonth);

        SetMarkedColor(&CC,3);
        GoToPosition(19,7);
        WriteInteger(DeviceInfo.MenuSetPlanValueDay);

        SetMarkedColor(&CC,4);
        GoToPosition(19,8);
        WriteInteger(DeviceInfo.MenuSetPlanValueHour);

        SetMarkedColor(&CC,5);
        GoToPosition(19,9);
        WriteInteger(DeviceInfo.MenuSetPlanValueMinute);

        SetMarkedColor(&CC,6);
        GoToPosition(19,10);
        WriteInteger(DeviceInfo.MenuSetPlanValueSecond);
    }

    if (DeviceInfo.MenuViewPlan != False)
    {
        TChar S[20];
        TInteger i;
        for (i = 0;i < ActionCount;i += 1)
        {
            WriteTime(S,DeviceInfo.CurrentPlans[i]);

            SetPlanSelectedColor(&CC,i);
            GoToPosition(5,2 + i);
            WriteString(S);
        }

        CC.Background = ccBlack;
        CC.Foreground = ccWhite;
        SetColor(&CC);

        GoToPosition(5,18);
        WriteString("Stergere (S)        Sus (Q)");

        GoToPosition(5,19);
        WriteString("Inapoi (I)          Jos (A)");
    }
}

int main(void)
{
    CreateConsole();
    SetTitle("Program de interfata");

    InitDevice(&DeviceInfo);

    InitNetworking(MyIP,CommPort);

    while (DeviceInfo.ToExit == False)
    {
        struct TConsoleColor CC;
        TInteger WX,WY;

        DeviceInfo.CurrentTime = GetTime();

        HandleInput(&DeviceInfo);

        HandleUploads();

        GetPacket();

        GetConsoleSize(&WX,&WY);

        CC.Foreground = ccWhite;
        CC.Background = ccBlack;
        ClearDisplay(WX,WY,&CC);

        DrawMenu();
        DrawWaterLevelStatus(DeviceInfo.WaterSensors);
        DrawDeviceStatus(&DeviceInfo);

        UpdateDisplay();
        WaitNanoSeconds(1000000);

        DeviceInfo.FrameCount += 1;
    }

    CloseNetworking();

    DestroyConsole();

    return 0;
}
